@@ -129,10 +129,9 @@ def percentile(a, q, interpolation="linear", method="default"):
         )
 
         name = "percentile_tdigest_chunk-" + token
-        dsk = dict(
-            ((name, i), (_tdigest_chunk, key))
-            for i, key in enumerate(a.__dask_keys__())
-        )
+        dsk = {
+            (name, i): (_tdigest_chunk, key) for i, key in enumerate(a.__dask_keys__())
+        }
 
         name2 = "percentile_tdigest-" + token
 
@@ -144,10 +143,10 @@ def percentile(a, q, interpolation="linear", method="default"):
         calc_q = np.pad(q, 1, mode="constant")
         calc_q[-1] = 100
         name = "percentile_chunk-" + token
-        dsk = dict(
-            ((name, i), (_percentile, key, calc_q, interpolation))
+        dsk = {
+            (name, i): (_percentile, key, calc_q, interpolation)
             for i, key in enumerate(a.__dask_keys__())
-        )
+        }
 
         name2 = "percentile-" + token
         dsk2 = {
@@ -165,7 +164,9 @@ def percentile(a, q, interpolation="linear", method="default"):
     return Array(graph, name2, chunks=((len(q),),), meta=meta)
 
 
-def merge_percentiles(finalq, qs, vals, interpolation="lower", Ns=None):
+def merge_percentiles(
+    finalq, qs, vals, interpolation="lower", Ns=None, raise_on_nan=True
+):
     """Combine several percentile calculations of different data.
 
     Parameters
@@ -207,14 +208,16 @@ def merge_percentiles(finalq, qs, vals, interpolation="lower", Ns=None):
 
     L = list(zip(*[(q, val, N) for q, val, N in zip(qs, vals, Ns) if N]))
     if not L:
-        raise ValueError("No non-trivial arrays found")
+        if raise_on_nan:
+            raise ValueError("No non-trivial arrays found")
+        return np.full(len(qs[0]) - 2, np.nan)
     qs, vals, Ns = L
 
     # TODO: Perform this check above in percentile once dtype checking is easy
     #       Here we silently change meaning
     if vals[0].dtype.name == "category":
         result = merge_percentiles(
-            finalq, qs, [v.codes for v in vals], interpolation, Ns
+            finalq, qs, [v.codes for v in vals], interpolation, Ns, raise_on_nan
         )
         import pandas as pd
 
