@@ -4,12 +4,12 @@ from operator import mul
 
 import numpy as np
 
-from .core import Array
-from .utils import meta_from_array
 from ..base import tokenize
 from ..core import flatten
 from ..highlevelgraph import HighLevelGraph
 from ..utils import M
+from .core import Array
+from .utils import meta_from_array
 
 
 def reshape_rechunk(inshape, outshape, inchunks):
@@ -49,8 +49,8 @@ def reshape_rechunk(inshape, outshape, inchunks):
             if all(len(inchunks[i]) == inshape[i] for i in range(ii)):
                 for i in range(ii + 1):
                     result_inchunks[i] = inchunks[i]
-                result_outchunks[oi] = inchunks[i] * np.prod(
-                    list(map(len, inchunks[:i]))
+                result_outchunks[oi] = inchunks[ii] * np.prod(
+                    list(map(len, inchunks[ileft:ii]))
                 )
             else:
                 for i in range(ileft + 1, ii + 1):  # need single-shape dimensions
@@ -143,9 +143,24 @@ def contract_tuple(chunks, factor):
     return tuple(out)
 
 
-def reshape(x, shape):
+def reshape(x, shape, merge_chunks=True):
     """Reshape array to new shape
 
+    Parameters
+    ----------
+    shape : int or tuple of ints
+        The new shape should be compatible with the original shape. If
+        an integer, then the result will be a 1-D array of that length.
+        One shape dimension can be -1. In this case, the value is
+        inferred from the length of the array and remaining dimensions.
+    merge_chunks : bool, default True
+        Whether to merge chunks using the logic in :meth:`dask.array.rechunk`
+        when communication is necessary given the input array chunking and
+        the output shape. With ``merge_chunks==False``, the input array will
+        be rechunked to a chunksize of 1, which can create very many tasks.
+
+    Notes
+    -----
     This is a parallelized version of the ``np.reshape`` function with the
     following limitations:
 
@@ -158,6 +173,9 @@ def reshape(x, shape):
     When communication is necessary this algorithm depends on the logic within
     rechunk.  It endeavors to keep chunk sizes roughly the same when possible.
 
+    See :ref:`array-chunks.reshaping` for a discussion the tradeoffs of
+    ``merge_chunks``.
+
     See Also
     --------
     dask.array.rechunk
@@ -181,7 +199,7 @@ def reshape(x, shape):
     if np.isnan(sum(x.shape)):
         raise ValueError(
             "Array chunk size or shape is unknown. shape: %s\n\n"
-            "Possible solution with x.compute_chunk_sizes()" % x.shape
+            "Possible solution with x.compute_chunk_sizes()" % str(x.shape)
         )
 
     if reduce(mul, shape, 1) != x.size:
@@ -201,7 +219,12 @@ def reshape(x, shape):
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[x])
         return Array(graph, name, chunks, meta=meta)
 
-    # Logic for how to rechunk
+    # Logic or how to rechunk
+    din = len(x.shape)
+    dout = len(shape)
+    if not merge_chunks and din > dout:
+        x = x.rechunk({i: 1 for i in range(din - dout)})
+
     inchunks, outchunks = reshape_rechunk(x.shape, shape, x.chunks)
     x2 = x.rechunk(inchunks)
 
