@@ -1,7 +1,9 @@
 import re
 
 import pandas as pd
+from fsspec.implementations.local import LocalFileSystem
 
+from .... import config
 from ....core import flatten
 from ....utils import natural_sort_key
 
@@ -672,3 +674,17 @@ def _get_aggregation_depth(aggregate_files, partition_names):
             )
 
     return aggregation_depth
+
+
+def _set_metadata_task_size(metadata_task_size, fs):
+    # Set metadata_task_size using the config file
+    # if the kwarg value was not specified
+    if metadata_task_size is None:
+        # If a default value is not specified in the config file,
+        # otherwise we use "0"
+        config_str = "dataframe.parquet.metadata-task-size-" + (
+            "local" if isinstance(fs, LocalFileSystem) else "remote"
+        )
+        return config.get(config_str, 0)
+
+    return metadata_task_size
