@@ -1,20 +1,17 @@
+from concurrent.futures import ThreadPoolExecutor
+from threading import Lock
+
 import numpy as np
 import pandas as pd
-
 import pytest
-from threading import Lock
-from multiprocessing.pool import ThreadPool
 
 import dask.array as da
 import dask.dataframe as dd
 from dask.dataframe._compat import tm
 from dask.dataframe.io.io import _meta_from_array
-from dask.delayed import Delayed, delayed
-
-from dask.utils import tmpfile
-
 from dask.dataframe.utils import assert_eq, is_categorical_dtype
-
+from dask.delayed import Delayed, delayed
+from dask.utils import tmpfile
 
 ####################
 # Arrays and BColz #
@@ -117,7 +114,6 @@ def test_from_array_with_record_dtype():
 
 def test_from_bcolz_multiple_threads():
     bcolz = pytest.importorskip("bcolz")
-    pool = ThreadPool(processes=5)
 
     def check(i):
         t = bcolz.ctable(
@@ -141,7 +137,8 @@ def test_from_bcolz_multiple_threads():
             dd.from_bcolz(t, chunksize=3).dask
         )
 
-    pool.map(check, range(5))
+    with ThreadPoolExecutor(5) as pool:
+        list(pool.map(check, range(5)))
 
 
 def test_from_bcolz():
@@ -507,7 +504,6 @@ def test_from_dask_array_unknown_chunks():
 
 
 def test_to_bag():
-    pytest.importorskip("dask.bag")
     a = pd.DataFrame(
         {"x": ["a", "b", "c", "d"], "y": [2, 3, 4, 5]},
         index=pd.Index([1.0, 2.0, 3.0, 4.0], name="ind"),
@@ -516,9 +512,34 @@ def test_to_bag():
 
     assert ddf.to_bag().compute() == list(a.itertuples(False))
     assert ddf.to_bag(True).compute() == list(a.itertuples(True))
+    assert ddf.to_bag(format="dict").compute() == [
+        {"x": "a", "y": 2},
+        {"x": "b", "y": 3},
+        {"x": "c", "y": 4},
+        {"x": "d", "y": 5},
+    ]
+    assert ddf.to_bag(True, format="dict").compute() == [
+        {"index": 1.0, "x": "a", "y": 2},
+        {"index": 2.0, "x": "b", "y": 3},
+        {"index": 3.0, "x": "c", "y": 4},
+        {"index": 4.0, "x": "d", "y": 5},
+    ]
     assert ddf.x.to_bag(True).compute() == list(a.x.iteritems())
     assert ddf.x.to_bag().compute() == list(a.x)
 
+    assert ddf.x.to_bag(True, format="dict").compute() == [
+        {"x": "a"},
+        {"x": "b"},
+        {"x": "c"},
+        {"x": "d"},
+    ]
+    assert ddf.x.to_bag(format="dict").compute() == [
+        {"x": "a"},
+        {"x": "b"},
+        {"x": "c"},
+        {"x": "d"},
+    ]
+
 
 def test_to_records():
     pytest.importorskip("dask.array")
@@ -530,7 +551,9 @@ def test_to_records():
     )
     ddf = dd.from_pandas(df, 2)
 
-    assert_eq(df.to_records(), ddf.to_records())
+    assert_eq(
+        df.to_records(), ddf.to_records(), check_type=False
+    )  # TODO: make check_type pass
 
 
 @pytest.mark.parametrize("lengths", [[2, 2], True])
@@ -545,7 +568,7 @@ def test_to_records_with_lengths(lengths):
     ddf = dd.from_pandas(df, 2)
 
     result = ddf.to_records(lengths=lengths)
-    assert_eq(df.to_records(), result)
+    assert_eq(df.to_records(), result, check_type=False)  # TODO: make check_type pass
 
     assert isinstance(result, da.Array)
 
@@ -602,6 +625,20 @@ def test_from_delayed():
     assert str(e.value).startswith("Metadata mismatch found in `from_delayed`")
 
 
+def test_from_delayed_preserves_hlgs():
+    df = pd.DataFrame(data=np.random.normal(size=(10, 4)), columns=list("abcd"))
+    parts = [df.iloc[:1], df.iloc[1:3], df.iloc[3:6], df.iloc[6:10]]
+    dfs = [delayed(parts.__getitem__)(i) for i in range(4)]
+    meta = dfs[0].compute()
+
+    chained = [d.a for d in dfs]
+    hlg = dd.from_delayed(chained, meta=meta).dask
+    for d in chained:
+        for layer_name, layer in d.dask.layers.items():
+            assert hlg.layers[layer_name] == layer
+            assert hlg.dependencies[layer_name] == d.dask.dependencies[layer_name]
+
+
 def test_from_delayed_misordered_meta():
     df = pd.DataFrame(
         columns=["(1)", "(2)", "date", "ent", "val"],
