@@ -2,17 +2,27 @@ import warnings
 
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_categorical_dtype, union_categoricals
 from tlz import partition
 
-from .utils import (
-    is_series_like,
-    is_index_like,
-    is_dataframe_like,
-    hash_object_dispatch,
+from ._compat import PANDAS_GT_131
+
+#  preserve compatibility while moving dispatch objects
+from .dispatch import (  # noqa: F401
+    concat,
+    concat_dispatch,
     group_split_dispatch,
+    hash_object_dispatch,
+    is_categorical_dtype,
+    is_categorical_dtype_dispatch,
+    tolist,
+    tolist_dispatch,
+    union_categoricals,
 )
-from ..utils import Dispatch
+from .utils import is_dataframe_like, is_index_like, is_series_like
+
+# cuDF may try to import old dispatch functions
+hash_df = hash_object_dispatch
+group_split = group_split_dispatch
 
 # ---------------------------------
 # indexing
@@ -43,9 +53,7 @@ def try_loc(df, iindexer, cindexer=None):
         return df.head(0).loc[:, cindexer]
 
 
-def boundary_slice(
-    df, start, stop, right_boundary=True, left_boundary=True, kind="loc"
-):
+def boundary_slice(df, start, stop, right_boundary=True, left_boundary=True, kind=None):
     """Index slice start/stop. Can switch include/exclude boundaries.
 
     Examples
@@ -80,6 +88,19 @@ def boundary_slice(
     if len(df.index) == 0:
         return df
 
+    if PANDAS_GT_131:
+        if kind is not None:
+            warnings.warn(
+                "The `kind` argument is no longer used/supported. "
+                "It will be dropped in a future release.",
+                category=FutureWarning,
+            )
+        kind_opts = {}
+        kind = "loc"
+    else:
+        kind = kind or "loc"
+        kind_opts = {"kind": kind}
+
     if kind == "loc" and not df.index.is_monotonic:
         # Pandas treats missing keys differently for label-slicing
         # on monotonic vs. non-monotonic indexes
@@ -96,13 +117,13 @@ def boundary_slice(
             else:
                 df = df[df.index < stop]
         return df
-    else:
-        result = getattr(df, kind)[start:stop]
+
+    result = getattr(df, kind)[start:stop]
     if not right_boundary and stop is not None:
-        right_index = result.index.get_slice_bound(stop, "left", kind)
+        right_index = result.index.get_slice_bound(stop, "left", **kind_opts)
         result = result.iloc[:right_index]
     if not left_boundary and start is not None:
-        left_index = result.index.get_slice_bound(start, "right", kind)
+        left_index = result.index.get_slice_bound(start, "right", **kind_opts)
         result = result.iloc[left_index:]
     return result
 
@@ -135,6 +156,13 @@ def wrap_skew_reduction(array_skew, index):
     return array_skew
 
 
+def wrap_kurtosis_reduction(array_kurtosis, index):
+    if isinstance(array_kurtosis, np.ndarray) or isinstance(array_kurtosis, list):
+        return pd.Series(array_kurtosis, index=index)
+
+    return array_kurtosis
+
+
 def var_mixed_concat(numeric_var, timedelta_var, columns):
     vars = pd.concat([numeric_var, timedelta_var])
 
@@ -155,7 +183,9 @@ def describe_aggregate(values):
     return pd.concat(values, axis=1, sort=False).reindex(names)
 
 
-def describe_numeric_aggregate(stats, name=None, is_timedelta_col=False):
+def describe_numeric_aggregate(
+    stats, name=None, is_timedelta_col=False, is_datetime_col=False
+):
     assert len(stats) == 6
     count, mean, std, min, q, max = stats
 
@@ -171,9 +201,18 @@ def describe_numeric_aggregate(stats, name=None, is_timedelta_col=False):
         max = pd.to_timedelta(max)
         q = q.apply(lambda x: pd.to_timedelta(x))
 
-    part1 = typ([count, mean, std, min], index=["count", "mean", "std", "min"])
+    if is_datetime_col:
+        # mean is not implemented for datetime
+        min = pd.to_datetime(min)
+        max = pd.to_datetime(max)
+        q = q.apply(lambda x: pd.to_datetime(x))
 
-    q.index = ["{0:g}%".format(l * 100) for l in tolist(q.index)]
+    if is_datetime_col:
+        part1 = typ([count, min], index=["count", "min"])
+    else:
+        part1 = typ([count, mean, std, min], index=["count", "mean", "std", "min"])
+
+    q.index = [f"{l * 100:g}%" for l in tolist(q.index)]
     if is_series_like(q) and typ != type(q):
         q = q.to_frame()
     part3 = typ([max], index=["max"])
@@ -309,8 +348,12 @@ def value_counts_combine(x, sort=True, ascending=False, **groupby_kwargs):
     return x.groupby(level=0, **groupby_kwargs).sum()
 
 
-def value_counts_aggregate(x, sort=True, ascending=False, **groupby_kwargs):
+def value_counts_aggregate(
+    x, sort=True, ascending=False, normalize=False, total_length=None, **groupby_kwargs
+):
     out = value_counts_combine(x, **groupby_kwargs)
+    if normalize:
+        out /= total_length if total_length is not None else out.sum()
     if sort:
         return out.sort_values(ascending=ascending)
     return out
@@ -373,214 +416,6 @@ def pivot_count(df, index, columns, values):
     ).astype(np.float64)
 
 
-# ---------------------------------
-# concat
-# ---------------------------------
-
-
-concat_dispatch = Dispatch("concat")
-
-
-def concat(
-    dfs,
-    axis=0,
-    join="outer",
-    uniform=False,
-    filter_warning=True,
-    ignore_index=False,
-    **kwargs
-):
-    """Concatenate, handling some edge cases:
-
-    - Unions categoricals between partitions
-    - Ignores empty partitions
-
-    Parameters
-    ----------
-    dfs : list of DataFrame, Series, or Index
-    axis : int or str, optional
-    join : str, optional
-    uniform : bool, optional
-        Whether to treat ``dfs[0]`` as representative of ``dfs[1:]``. Set to
-        True if all arguments have the same columns and dtypes (but not
-        necessarily categories). Default is False.
-    ignore_index : bool, optional
-        Whether to allow index values to be ignored/droped during
-        concatenation. Default is False.
-    """
-    if len(dfs) == 1:
-        return dfs[0]
-    else:
-        func = concat_dispatch.dispatch(type(dfs[0]))
-        return func(
-            dfs,
-            axis=axis,
-            join=join,
-            uniform=uniform,
-            filter_warning=filter_warning,
-            ignore_index=ignore_index,
-            **kwargs
-        )
-
-
-@concat_dispatch.register((pd.DataFrame, pd.Series, pd.Index))
-def concat_pandas(
-    dfs,
-    axis=0,
-    join="outer",
-    uniform=False,
-    filter_warning=True,
-    ignore_index=False,
-    **kwargs
-):
-    if axis == 1:
-        return pd.concat(dfs, axis=axis, join=join, **kwargs)
-
-    # Support concatenating indices along axis 0
-    if isinstance(dfs[0], pd.Index):
-        if isinstance(dfs[0], pd.CategoricalIndex):
-            for i in range(1, len(dfs)):
-                if not isinstance(dfs[i], pd.CategoricalIndex):
-                    dfs[i] = dfs[i].astype("category")
-            return pd.CategoricalIndex(union_categoricals(dfs), name=dfs[0].name)
-        elif isinstance(dfs[0], pd.MultiIndex):
-            first, rest = dfs[0], dfs[1:]
-            if all(
-                (isinstance(o, pd.MultiIndex) and o.nlevels >= first.nlevels)
-                for o in rest
-            ):
-                arrays = [
-                    concat([i._get_level_values(n) for i in dfs])
-                    for n in range(first.nlevels)
-                ]
-                return pd.MultiIndex.from_arrays(arrays, names=first.names)
-
-            to_concat = (first.values,) + tuple(k._values for k in rest)
-            new_tuples = np.concatenate(to_concat)
-            try:
-                return pd.MultiIndex.from_tuples(new_tuples, names=first.names)
-            except Exception:
-                return pd.Index(new_tuples)
-        return dfs[0].append(dfs[1:])
-
-    # Handle categorical index separately
-    dfs0_index = dfs[0].index
-
-    has_categoricalindex = isinstance(dfs0_index, pd.CategoricalIndex) or (
-        isinstance(dfs0_index, pd.MultiIndex)
-        and any(isinstance(i, pd.CategoricalIndex) for i in dfs0_index.levels)
-    )
-
-    if has_categoricalindex:
-        dfs2 = [df.reset_index(drop=True) for df in dfs]
-        ind = concat([df.index for df in dfs])
-    else:
-        dfs2 = dfs
-        ind = None
-
-    # Concatenate the partitions together, handling categories as needed
-    if (
-        isinstance(dfs2[0], pd.DataFrame)
-        if uniform
-        else any(isinstance(df, pd.DataFrame) for df in dfs2)
-    ):
-        if uniform:
-            dfs3 = dfs2
-            cat_mask = dfs2[0].dtypes == "category"
-        else:
-            # When concatenating mixed dataframes and series on axis 1, Pandas
-            # converts series to dataframes with a single column named 0, then
-            # concatenates.
-            dfs3 = [
-                df
-                if isinstance(df, pd.DataFrame)
-                else df.to_frame().rename(columns={df.name: 0})
-                for df in dfs2
-            ]
-            # pandas may raise a RuntimeWarning for comparing ints and strs
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore", RuntimeWarning)
-                if filter_warning:
-                    warnings.simplefilter("ignore", FutureWarning)
-                cat_mask = pd.concat(
-                    [(df.dtypes == "category").to_frame().T for df in dfs3],
-                    join=join,
-                    **kwargs
-                ).any()
-
-        if cat_mask.any():
-            not_cat = cat_mask[~cat_mask].index
-            # this should be aligned, so no need to filter warning
-            out = pd.concat(
-                [df[df.columns.intersection(not_cat)] for df in dfs3],
-                join=join,
-                **kwargs
-            )
-            temp_ind = out.index
-            for col in cat_mask.index.difference(not_cat):
-                # Find an example of categoricals in this column
-                for df in dfs3:
-                    sample = df.get(col)
-                    if sample is not None:
-                        break
-                # Extract partitions, subbing in missing if needed
-                parts = []
-                for df in dfs3:
-                    if col in df.columns:
-                        parts.append(df[col])
-                    else:
-                        codes = np.full(len(df), -1, dtype="i8")
-                        data = pd.Categorical.from_codes(
-                            codes, sample.cat.categories, sample.cat.ordered
-                        )
-                        parts.append(data)
-                out[col] = union_categoricals(parts)
-                # Pandas resets index type on assignment if frame is empty
-                # https://github.com/pandas-dev/pandas/issues/17101
-                if not len(temp_ind):
-                    out.index = temp_ind
-            out = out.reindex(columns=cat_mask.index)
-        else:
-            # pandas may raise a RuntimeWarning for comparing ints and strs
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore", RuntimeWarning)
-                if filter_warning:
-                    warnings.simplefilter("ignore", FutureWarning)
-                out = pd.concat(dfs3, join=join, sort=False)
-    else:
-        if is_categorical_dtype(dfs2[0].dtype):
-            if ind is None:
-                ind = concat([df.index for df in dfs2])
-            return pd.Series(union_categoricals(dfs2), index=ind, name=dfs2[0].name)
-        with warnings.catch_warnings():
-            if filter_warning:
-                warnings.simplefilter("ignore", FutureWarning)
-
-            out = pd.concat(dfs2, join=join, **kwargs)
-    # Re-add the index if needed
-    if ind is not None:
-        out.index = ind
-    return out
-
-
-tolist_dispatch = Dispatch("tolist")
-
-
-def tolist(obj):
-    func = tolist_dispatch.dispatch(type(obj))
-    return func(obj)
-
-
-@tolist_dispatch.register((pd.Series, pd.Index, pd.Categorical))
-def tolist_pandas(obj):
-    return obj.tolist()
-
-
-# cuDF may try to import old dispatch functions
-hash_df = hash_object_dispatch
-group_split = group_split_dispatch
-
-
 def assign_index(df, ind):
     df = df.copy()
     df.index = ind
