@@ -53,49 +53,44 @@ We proceed with hash joins in the following stages:
     ``dask.dataframe.shuffle.shuffle``.
 2.  Perform embarrassingly parallel join across shuffled inputs.
 """
-from functools import wraps, partial
-import warnings
 import math
-import operator
-from collections import defaultdict
 import pickle
+import warnings
+from functools import partial, wraps
 
-from tlz import merge_sorted, unique, first, valmap
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_dtype_equal, is_categorical_dtype, union_categoricals
+from pandas.api.types import is_categorical_dtype, is_dtype_equal
+from tlz import first, merge_sorted, unique
 
-from ..core import keys_in_tasks
-from ..base import tokenize, is_dask_collection
-from ..highlevelgraph import HighLevelGraph, Layer
-from ..utils import apply, stringify, stringify_collection_keys
-from ._compat import PANDAS_GT_100
+from ..base import is_dask_collection, tokenize
+from ..highlevelgraph import HighLevelGraph
+from ..layers import BroadcastJoinLayer
+from ..utils import M, apply
+from . import methods
 from .core import (
-    _Frame,
     DataFrame,
-    Series,
-    map_partitions,
     Index,
+    Series,
+    _concat,
+    _Frame,
     _maybe_from_pandas,
-    new_dd_object,
     is_broadcastable,
+    map_partitions,
+    new_dd_object,
     prefix_reduction,
     suffix_reduction,
-    _concat,
 )
+from .dispatch import group_split_dispatch, hash_object_dispatch
 from .io import from_pandas
-from . import methods
-from .shuffle import shuffle, rearrange_by_divisions, partitioning_index, shuffle_group
+from .shuffle import partitioning_index, rearrange_by_divisions, shuffle, shuffle_group
 from .utils import (
-    strip_unknown_categories,
-    is_series_like,
     asciitable,
     is_dataframe_like,
+    is_series_like,
     make_meta,
-    hash_object_dispatch,
-    group_split_dispatch,
+    strip_unknown_categories,
 )
-from ..utils import M
 
 
 def align_partitions(*dfs):
@@ -245,7 +240,7 @@ def merge_chunk(lhs, *args, **kwargs):
     left_index = kwargs.get("left_index", False)
     right_index = kwargs.get("right_index", False)
 
-    if categorical_columns is not None and PANDAS_GT_100:
+    if categorical_columns is not None:
         for col in categorical_columns:
             left = None
             right = None
@@ -264,8 +259,8 @@ def merge_chunk(lhs, *args, **kwargs):
 
             dtype = "category"
             if left is not None and right is not None:
-                dtype = union_categoricals(
-                    [left.astype("category").values, right.astype("category").values]
+                dtype = methods.union_categoricals(
+                    [left.astype("category"), right.astype("category")]
                 ).dtype
 
             if left is not None:
@@ -290,7 +285,7 @@ def merge_chunk(lhs, *args, **kwargs):
 
 
 def merge_indexed_dataframes(lhs, rhs, left_index=True, right_index=True, **kwargs):
-    """ Join two partitioned dataframes along their index """
+    """Join two partitioned dataframes along their index"""
     how = kwargs.get("how", "left")
     kwargs["left_index"] = left_index
     kwargs["right_index"] = right_index
@@ -332,7 +327,7 @@ def hash_join(
     This shuffles both datasets on the joined column and then performs an
     embarrassingly parallel join partition-by-partition
 
-    >>> hash_join(a, 'id', rhs, 'id', how='left', npartitions=10)  # doctest: +SKIP
+    >>> hash_join(lhs, 'id', rhs, 'id', how='left', npartitions=10)  # doctest: +SKIP
     """
     if npartitions is None:
         npartitions = max(lhs.npartitions, rhs.npartitions)
@@ -367,7 +362,10 @@ def hash_join(
     )
 
     # dummy result
-    meta = lhs._meta_nonempty.merge(rhs._meta_nonempty, **kwargs)
+    # Avoid using dummy data for a collection it is empty
+    _lhs_meta = lhs._meta_nonempty if len(lhs.columns) else lhs._meta
+    _rhs_meta = rhs._meta_nonempty if len(rhs.columns) else rhs._meta
+    meta = _lhs_meta.merge(_rhs_meta, **kwargs)
 
     if isinstance(left_on, list):
         left_on = (list, tuple(left_on))
@@ -395,35 +393,54 @@ def single_partition_join(left, right, **kwargs):
     # new index will not necessarily correspond with the current divisions
 
     meta = left._meta_nonempty.merge(right._meta_nonempty, **kwargs)
+
+    use_left = kwargs.get("right_index") or right._contains_index_name(
+        kwargs.get("right_on")
+    )
+    use_right = kwargs.get("left_index") or left._contains_index_name(
+        kwargs.get("left_on")
+    )
+
+    if len(meta) == 0:
+        if use_left:
+            meta.index = meta.index.astype(left.index.dtype)
+        elif use_right:
+            meta.index = meta.index.astype(right.index.dtype)
+        else:
+            meta.index = meta.index.astype("int64")
+
     kwargs["empty_index_dtype"] = meta.index.dtype
     kwargs["categorical_columns"] = meta.select_dtypes(include="category").columns
 
     name = "merge-" + tokenize(left, right, **kwargs)
-    if left.npartitions == 1 and kwargs["how"] in allowed_right:
+
+    if right.npartitions == 1 and kwargs["how"] in allowed_left:
+        right_key = first(right.__dask_keys__())
+        dsk = {
+            (name, i): (apply, merge_chunk, [left_key, right_key], kwargs)
+            for i, left_key in enumerate(left.__dask_keys__())
+        }
+        if use_left:
+            divisions = left.divisions
+        elif use_right and len(right.divisions) == len(left.divisions):
+            divisions = right.divisions
+        else:
+            divisions = [None for _ in left.divisions]
+
+    elif left.npartitions == 1 and kwargs["how"] in allowed_right:
         left_key = first(left.__dask_keys__())
         dsk = {
             (name, i): (apply, merge_chunk, [left_key, right_key], kwargs)
             for i, right_key in enumerate(right.__dask_keys__())
         }
 
-        if kwargs.get("right_index") or right._contains_index_name(
-            kwargs.get("right_on")
-        ):
+        if use_right:
             divisions = right.divisions
+        elif use_left and len(left.divisions) == len(right.divisions):
+            divisions = left.divisions
         else:
             divisions = [None for _ in right.divisions]
 
-    elif right.npartitions == 1 and kwargs["how"] in allowed_left:
-        right_key = first(right.__dask_keys__())
-        dsk = {
-            (name, i): (apply, merge_chunk, [left_key, right_key], kwargs)
-            for i, left_key in enumerate(left.__dask_keys__())
-        }
-
-        if kwargs.get("left_index") or left._contains_index_name(kwargs.get("left_on")):
-            divisions = left.divisions
-        else:
-            divisions = [None for _ in left.divisions]
     else:
         raise NotImplementedError(
             "single_partition_join has no fallback for invalid calls"
@@ -515,14 +532,14 @@ def merge(
     if not is_dask_collection(left):
         if right_index and left_on:  # change to join on index
             left = left.set_index(left[left_on])
-            left_on = False
+            left_on = None
             left_index = True
         left = from_pandas(left, npartitions=1)  # turn into DataFrame
 
     if not is_dask_collection(right):
         if left_index and right_on:  # change to join on index
             right = right.set_index(right[right_on])
-            right_on = False
+            right_on = None
             right_index = True
         right = from_pandas(right, npartitions=1)  # turn into DataFrame
 
@@ -771,7 +788,7 @@ def pair_partitions(L, R):
 
 
 def merge_asof_padded(left, right, prev=None, next=None, **kwargs):
-    """ merge_asof but potentially adding rows to the beginning/end of right """
+    """merge_asof but potentially adding rows to the beginning/end of right"""
     frames = []
     if prev is not None:
         frames.append(prev)
@@ -804,33 +821,27 @@ def get_unsorted_columns(frames):
     return order
 
 
-def _concat_compat(frames, left, right):
-    if PANDAS_GT_100:
-        # join_axes removed
-        return (pd.concat, frames, 0, "outer", False, None, None, None, False, False)
-    else:
-        # (axis, join, join_axis, ignore_index, keys, levels, names, verify_integrity, sort)
-        # we only care about sort, to silence warnings.
-        return (
-            pd.concat,
-            frames,
-            0,
-            "outer",
-            None,
-            False,
-            None,
-            None,
-            None,
-            False,
-            False,
-        )
-
-
 def merge_asof_indexed(left, right, **kwargs):
     dsk = dict()
     name = "asof-join-indexed-" + tokenize(left, right, **kwargs)
     meta = pd.merge_asof(left._meta_nonempty, right._meta_nonempty, **kwargs)
 
+    if all(map(pd.isnull, left.divisions)):
+        # results in an empty df that looks like ``meta``
+        return from_pandas(meta.iloc[len(meta) :], npartitions=left.npartitions)
+
+    if all(map(pd.isnull, right.divisions)):
+        # results in an df that looks like ``left`` with nulls for
+        # all ``right.columns``
+        return map_partitions(
+            pd.merge_asof,
+            left,
+            right=right,
+            left_index=True,
+            right_index=True,
+            meta=meta,
+        )
+
     dependencies = [left, right]
     tails = heads = None
     if kwargs["direction"] in ["backward", "nearest"]:
@@ -854,8 +865,7 @@ def merge_asof_indexed(left, right, **kwargs):
                     kwargs,
                 )
             )
-        args = _concat_compat(frames, left, right)
-        dsk[(name, i)] = args
+        dsk[(name, i)] = (methods.concat, frames)
 
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=dependencies)
     result = new_dd_object(graph, name, meta, left.divisions)
@@ -901,7 +911,7 @@ def merge_asof(
     }
 
     if left is None or right is None:
-        raise ValueError("Cannot merge_asof on empty DataFrames")
+        raise ValueError("Cannot merge_asof on None")
 
     # if is_dataframe_like(left) and is_dataframe_like(right):
     if isinstance(left, pd.DataFrame) and isinstance(right, pd.DataFrame):
@@ -958,31 +968,36 @@ def merge_asof(
 ###############################################################
 
 
-def concat_and_check(dfs):
+def concat_and_check(dfs, ignore_order=False):
     if len(set(map(len, dfs))) != 1:
         raise ValueError("Concatenated DataFrames of different lengths")
-    return methods.concat(dfs, axis=1)
+    return methods.concat(dfs, axis=1, ignore_order=ignore_order)
 
 
-def concat_unindexed_dataframes(dfs, **kwargs):
+def concat_unindexed_dataframes(dfs, ignore_order=False, **kwargs):
     name = "concat-" + tokenize(*dfs)
 
     dsk = {
-        (name, i): (concat_and_check, [(df._name, i) for df in dfs])
+        (name, i): (concat_and_check, [(df._name, i) for df in dfs], ignore_order)
         for i in range(dfs[0].npartitions)
     }
-
+    kwargs.update({"ignore_order": ignore_order})
     meta = methods.concat([df._meta for df in dfs], axis=1, **kwargs)
 
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=dfs)
     return new_dd_object(graph, name, meta, dfs[0].divisions)
 
 
-def concat_indexed_dataframes(dfs, axis=0, join="outer", **kwargs):
-    """ Concatenate indexed dataframes together along the index """
+def concat_indexed_dataframes(dfs, axis=0, join="outer", ignore_order=False, **kwargs):
+    """Concatenate indexed dataframes together along the index"""
     warn = axis != 0
+    kwargs.update({"ignore_order": ignore_order})
     meta = methods.concat(
-        [df._meta for df in dfs], axis=axis, join=join, filter_warning=warn, **kwargs
+        [df._meta for df in dfs],
+        axis=axis,
+        join=join,
+        filter_warning=warn,
+        **kwargs,
     )
     empties = [strip_unknown_categories(df._meta) for df in dfs]
 
@@ -998,28 +1013,34 @@ def concat_indexed_dataframes(dfs, axis=0, join="outer", **kwargs):
     filter_warning = True
     uniform = False
 
-    dsk = dict(
-        ((name, i), (methods.concat, part, axis, join, uniform, filter_warning))
+    dsk = {
+        (name, i): (methods.concat, part, axis, join, uniform, filter_warning, kwargs)
         for i, part in enumerate(parts2)
-    )
+    }
     for df in dfs2:
         dsk.update(df.dask)
 
     return new_dd_object(dsk, name, meta, divisions)
 
 
-def stack_partitions(dfs, divisions, join="outer", **kwargs):
+def stack_partitions(dfs, divisions, join="outer", ignore_order=False, **kwargs):
     """Concatenate partitions on axis=0 by doing a simple stack"""
     # Use _meta_nonempty as pandas.concat will incorrectly cast float to datetime
     # for empty data frames. See https://github.com/pandas-dev/pandas/issues/32934.
+
+    kwargs.update({"ignore_order": ignore_order})
+
     meta = make_meta(
         methods.concat(
-            [df._meta_nonempty for df in dfs], join=join, filter_warning=False, **kwargs
+            [df._meta_nonempty for df in dfs],
+            join=join,
+            filter_warning=False,
+            **kwargs,
         )
     )
     empty = strip_unknown_categories(meta)
 
-    name = "concat-{0}".format(tokenize(*dfs))
+    name = f"concat-{tokenize(*dfs)}"
     dsk = {}
     i = 0
     for df in dfs:
@@ -1056,11 +1077,19 @@ def stack_partitions(dfs, divisions, join="outer", **kwargs):
         except (ValueError, TypeError):
             match = False
 
+        filter_warning = True
+        uniform = False
+
         for key in df.__dask_keys__():
             if match:
                 dsk[(name, i)] = key
             else:
-                dsk[(name, i)] = (methods.concat, [empty, key], 0, join)
+                dsk[(name, i)] = (
+                    apply,
+                    methods.concat,
+                    [[empty, key], 0, join, uniform, filter_warning],
+                    kwargs,
+                )
             i += 1
 
     return new_dd_object(dsk, name, meta, divisions)
@@ -1072,6 +1101,7 @@ def concat(
     join="outer",
     interleave_partitions=False,
     ignore_unknown_divisions=False,
+    ignore_order=False,
     **kwargs,
 ):
     """Concatenate DataFrames along rows.
@@ -1105,6 +1135,8 @@ def concat(
     ignore_unknown_divisions : bool, default False
         By default a warning is raised if any input has unknown divisions.
         Set to True to disable this warning.
+    ignore_order : bool, default False
+        Whether to ignore order when doing the union of categoricals.
 
     Notes
     -----
@@ -1118,6 +1150,7 @@ def concat(
     --------
     If all divisions are known and ordered, divisions are kept.
 
+    >>> import dask.dataframe as dd
     >>> a                                               # doctest: +SKIP
     dd.DataFrame<x, divisions=(1, 3, 5)>
     >>> b                                               # doctest: +SKIP
@@ -1157,12 +1190,13 @@ def concat(
 
     Different categoricals are unioned
 
-    >> dd.concat([                                     # doctest: +SKIP
+    >>> dd.concat([
     ...     dd.from_pandas(pd.Series(['a', 'b'], dtype='category'), 1),
     ...     dd.from_pandas(pd.Series(['a', 'c'], dtype='category'), 1),
     ... ], interleave_partitions=True).dtype
     CategoricalDtype(categories=['a', 'b', 'c'], ordered=False)
     """
+
     if not isinstance(dfs, list):
         raise TypeError("dfs must be a list of DataFrames/Series objects")
     if len(dfs) == 0:
@@ -1182,7 +1216,9 @@ def concat(
 
     if axis == 1:
         if all(df.known_divisions for df in dasks):
-            return concat_indexed_dataframes(dfs, axis=axis, join=join, **kwargs)
+            return concat_indexed_dataframes(
+                dfs, axis=axis, join=join, ignore_order=ignore_order, **kwargs
+            )
         elif (
             len(dasks) == len(dfs)
             and all(not df.known_divisions for df in dfs)
@@ -1195,7 +1231,7 @@ def concat(
                     " are \n aligned. This assumption is not generally "
                     "safe."
                 )
-            return concat_unindexed_dataframes(dfs, **kwargs)
+            return concat_unindexed_dataframes(dfs, ignore_order=ignore_order, **kwargs)
         else:
             raise ValueError(
                 "Unable to concatenate DataFrame with unknown "
@@ -1213,298 +1249,23 @@ def concat(
                     # remove last to concatenate with next
                     divisions += df.divisions[:-1]
                 divisions += dfs[-1].divisions
-                return stack_partitions(dfs, divisions, join=join, **kwargs)
+                return stack_partitions(
+                    dfs, divisions, join=join, ignore_order=ignore_order, **kwargs
+                )
             elif interleave_partitions:
-                return concat_indexed_dataframes(dfs, join=join, **kwargs)
-            else:
-                divisions = [None] * (sum([df.npartitions for df in dfs]) + 1)
-                return stack_partitions(dfs, divisions, join=join, **kwargs)
-        else:
-            divisions = [None] * (sum([df.npartitions for df in dfs]) + 1)
-            return stack_partitions(dfs, divisions, join=join, **kwargs)
-
-
-class BroadcastJoinLayer(Layer):
-    """Broadcast-based Join Layer
-
-    High-level graph layer for a join operation requiring the
-    smaller collection to be broadcasted to every partition of
-    the larger collection.
-
-    Parameters
-    ----------
-    name : str
-        Name of new (joined) output collection.
-    lhs_name: string
-        "Left" DataFrame collection to join.
-    lhs_npartitions: int
-        Number of partitions in "left" DataFrame collection.
-    rhs_name: string
-        "Right" DataFrame collection to join.
-    rhs_npartitions: int
-        Number of partitions in "right" DataFrame collection.
-    parts_out : list of int (optional)
-        List of required output-partition indices.
-    annotations : dict (optional)
-        Layer annotations.
-    **merge_kwargs : **dict
-        Keyword arguments to be passed to chunkwise merge func.
-    """
-
-    def __init__(
-        self,
-        name,
-        npartitions,
-        lhs_name,
-        lhs_npartitions,
-        rhs_name,
-        rhs_npartitions,
-        parts_out=None,
-        annotations=None,
-        **merge_kwargs,
-    ):
-        super().__init__(annotations=annotations)
-        self.name = name
-        self.npartitions = npartitions
-        self.lhs_name = lhs_name
-        self.lhs_npartitions = lhs_npartitions
-        self.rhs_name = rhs_name
-        self.rhs_npartitions = rhs_npartitions
-        self.parts_out = parts_out or set(range(self.npartitions))
-        self.merge_kwargs = merge_kwargs
-        self.how = self.merge_kwargs.get("how")
-        self.left_on = self.merge_kwargs.get("left_on")
-        self.right_on = self.merge_kwargs.get("right_on")
-        if isinstance(self.left_on, list):
-            self.left_on = (list, tuple(self.left_on))
-        if isinstance(self.right_on, list):
-            self.right_on = (list, tuple(self.right_on))
-
-    def get_output_keys(self):
-        return {(self.name, part) for part in self.parts_out}
-
-    def __repr__(self):
-        return "BroadcastJoinLayer<name='{}', how={}, lhs={}, rhs={}>".format(
-            self.name, self.how, self.lhs_name, self.rhs_name
-        )
-
-    def is_materialized(self):
-        return hasattr(self, "_cached_dict")
-
-    @property
-    def _dict(self):
-        """Materialize full dict representation"""
-        if hasattr(self, "_cached_dict"):
-            return self._cached_dict
-        else:
-            dsk = self._construct_graph()
-            self._cached_dict = dsk
-        return self._cached_dict
-
-    def __getitem__(self, key):
-        return self._dict[key]
-
-    def __iter__(self):
-        return iter(self._dict)
-
-    def __len__(self):
-        return len(self._dict)
-
-    def __dask_distributed_pack__(self, client):
-
-        # Pickle complex merge_kwargs elements. Also
-        # tuples, which may be confused with keys.
-        _merge_kwargs = {}
-        for k, v in self.merge_kwargs.items():
-            if not isinstance(v, (str, list, bool)):
-                _merge_kwargs[k] = pickle.dumps(v)
-            else:
-                _merge_kwargs[k] = v
-
-        return {
-            "name": self.name,
-            "npartitions": self.npartitions,
-            "lhs_name": self.lhs_name,
-            "lhs_npartitions": self.lhs_npartitions,
-            "rhs_name": self.rhs_name,
-            "rhs_npartitions": self.rhs_npartitions,
-            "parts_out": self.parts_out,
-            "annotations": self.pack_annotations(),
-            "merge_kwargs": _merge_kwargs,
-        }
-
-    @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies, annotations):
-        from distributed.worker import dumps_task
-
-        # Expand merge_kwargs
-        merge_kwargs = state.pop("merge_kwargs", {})
-        state.update(merge_kwargs)
-
-        # Materialize the layer
-        raw = dict(cls(**state))
-
-        # Convert all keys to strings and dump tasks
-        raw = {stringify(k): stringify_collection_keys(v) for k, v in raw.items()}
-        dsk.update(valmap(dumps_task, raw))
-
-        dependencies.update(
-            {k: keys_in_tasks(dsk, [v], as_list=True) for k, v in raw.items()}
-        )
-
-        if state["annotations"]:
-            cls.unpack_annotations(annotations, state["annotations"], raw.keys())
-
-    def _keys_to_parts(self, keys):
-        """Simple utility to convert keys to partition indices."""
-        parts = set()
-        for key in keys:
-            try:
-                _name, _part = key
-            except ValueError:
-                continue
-            if _name != self.name:
-                continue
-            parts.add(_part)
-        return parts
-
-    @property
-    def _broadcast_plan(self):
-        # Return structure (tuple):
-        # (
-        #     <broadcasted-collection-name>,
-        #     <broadcasted-collection-npartitions>,
-        #     <other-collection-npartitions>,
-        #     <other-collection-on>,
-        # )
-        if self.lhs_npartitions < self.rhs_npartitions:
-            # Broadcasting the left
-            return (
-                self.lhs_name,
-                self.lhs_npartitions,
-                self.rhs_name,
-                self.right_on,
-            )
-        else:
-            # Broadcasting the right
-            return (
-                self.rhs_name,
-                self.rhs_npartitions,
-                self.lhs_name,
-                self.left_on,
-            )
-
-    def _cull_dependencies(self, keys, parts_out=None):
-        """Determine the necessary dependencies to produce `keys`.
-
-        For a broadcast join, output partitions always depend on
-        all partitions of the broadcasted collection, but only one
-        partition of the "other" collecction.
-        """
-        # Get broadcast info
-        bcast_name, bcast_size, other_name = self._broadcast_plan[:3]
-
-        deps = defaultdict(set)
-        parts_out = parts_out or self._keys_to_parts(keys)
-        for part in parts_out:
-            deps[(self.name, part)] |= {(bcast_name, i) for i in range(bcast_size)}
-            deps[(self.name, part)] |= {
-                (other_name, part),
-            }
-        return deps
-
-    def _cull(self, parts_out):
-        return BroadcastJoinLayer(
-            self.name,
-            self.npartitions,
-            self.lhs_name,
-            self.lhs_npartitions,
-            self.rhs_name,
-            self.rhs_npartitions,
-            self.merge_kwargs,
-            annotations=self.annotations,
-            parts_out=parts_out,
-        )
-
-    def cull(self, keys, all_keys):
-        """Cull a BroadcastJoinLayer HighLevelGraph layer.
-
-        The underlying graph will only include the necessary
-        tasks to produce the keys (indicies) included in `parts_out`.
-        Therefore, "culling" the layer only requires us to reset this
-        parameter.
-        """
-        parts_out = self._keys_to_parts(keys)
-        culled_deps = self._cull_dependencies(keys, parts_out=parts_out)
-        if parts_out != set(self.parts_out):
-            culled_layer = self._cull(parts_out)
-            return culled_layer, culled_deps
-        else:
-            return self, culled_deps
-
-    def _construct_graph(self):
-        """Construct graph for a broadcast join operation."""
-
-        inter_name = "inter-" + self.name
-        split_name = "split-" + self.name
-
-        # Get broadcast "plan"
-        bcast_name, bcast_size, other_name, other_on = self._broadcast_plan
-        bcast_side = "left" if self.lhs_npartitions < self.rhs_npartitions else "right"
-
-        # Loop over output partitions, which should be a 1:1
-        # mapping with the input partitions of "other".
-        # Culling should allow us to avoid generating tasks for
-        # any output partitions that are not requested (via `parts_out`)
-        dsk = {}
-        for i in self.parts_out:
-
-            # Split each "other" partition by hash
-            if self.how != "inner":
-                dsk[(split_name, i)] = (
-                    _split_partition,
-                    (other_name, i),
-                    other_on,
-                    bcast_size,
+                return concat_indexed_dataframes(
+                    dfs, join=join, ignore_order=ignore_order, **kwargs
                 )
-
-            # For each partition of "other", we need to join
-            # to each partition of "bcast". If it is a "left"
-            # or "right" join, there should be a unique mapping
-            # between the local splits of "other" and the
-            # partitions of "bcast" (which means we need an
-            # additional `getitem` operation to isolate the
-            # correct split of each "other" partition).
-            _concat_list = []
-            for j in range(bcast_size):
-                # Specify arg list for `merge_chunk`
-                _merge_args = [
-                    (
-                        operator.getitem,
-                        (split_name, i),
-                        j,
-                    )
-                    if self.how != "inner"
-                    else (other_name, i),
-                    (bcast_name, j),
-                ]
-                if bcast_side == "left":
-                    # If the left is broadcasted, the
-                    # arg list needs to be reversed
-                    _merge_args.reverse()
-                inter_key = (inter_name, i, j)
-                dsk[inter_key] = (
-                    apply,
-                    _merge_chunk_wrapper,
-                    _merge_args,
-                    self.merge_kwargs,
+            else:
+                divisions = [None] * (sum(df.npartitions for df in dfs) + 1)
+                return stack_partitions(
+                    dfs, divisions, join=join, ignore_order=ignore_order, **kwargs
                 )
-                _concat_list.append(inter_key)
-
-            # Concatenate the merged results for each output partition
-            dsk[(self.name, i)] = (_concat_wrapper, _concat_list)
-
-        return dsk
+        else:
+            divisions = [None] * (sum(df.npartitions for df in dfs) + 1)
+            return stack_partitions(
+                dfs, divisions, join=join, ignore_order=ignore_order, **kwargs
+            )
 
 
 def _contains_index_name(df, columns_or_index):
@@ -1753,3 +1514,55 @@ def broadcast_join(
     )
 
     return new_dd_object(graph, name, meta, divisions)
+
+
+def _recursive_pairwise_outer_join(
+    dataframes_to_merge, on, lsuffix, rsuffix, npartitions, shuffle
+):
+    """
+    Schedule the merging of a list of dataframes in a pairwise method. This is a recursive function that results
+    in a much more efficient scheduling of merges than a simple loop
+    from:
+    [A] [B] [C] [D] -> [AB] [C] [D] -> [ABC] [D] -> [ABCD]
+    to:
+    [A] [B] [C] [D] -> [AB] [CD] -> [ABCD]
+    Note that either way, n-1 merges are still required, but using a pairwise reduction it can be completed in parallel.
+    :param dataframes_to_merge: A list of Dask dataframes to be merged together on their index
+    :return: A single Dask Dataframe, comprised of the pairwise-merges of all provided dataframes
+    """
+    number_of_dataframes_to_merge = len(dataframes_to_merge)
+
+    merge_options = {
+        "on": on,
+        "lsuffix": lsuffix,
+        "rsuffix": rsuffix,
+        "npartitions": npartitions,
+        "shuffle": shuffle,
+    }
+
+    # Base case 1: just return the provided dataframe and merge with `left`
+    if number_of_dataframes_to_merge == 1:
+        return dataframes_to_merge[0]
+
+    # Base case 2: merge the two provided dataframe to be merged with `left`
+    if number_of_dataframes_to_merge == 2:
+        merged_ddf = dataframes_to_merge[0].join(
+            dataframes_to_merge[1], how="outer", **merge_options
+        )
+        return merged_ddf
+
+    # Recursive case: split the list of dfs into two ~even sizes and continue down
+    else:
+        middle_index = number_of_dataframes_to_merge // 2
+        merged_ddf = _recursive_pairwise_outer_join(
+            [
+                _recursive_pairwise_outer_join(
+                    dataframes_to_merge[:middle_index], **merge_options
+                ),
+                _recursive_pairwise_outer_join(
+                    dataframes_to_merge[middle_index:], **merge_options
+                ),
+            ],
+            **merge_options,
+        )
+        return merged_ddf
