@@ -3,19 +3,23 @@ import types
 import uuid
 import warnings
 from collections.abc import Iterator
+from dataclasses import fields, is_dataclass
 
-from tlz import curry, concat, unique, merge
+from tlz import concat, curry, merge, unique
 
 from . import config, threaded
-from .base import is_dask_collection, dont_optimize, DaskMethodsMixin
-from .base import replace_name_in_key, tokenize as _tokenize
-from .compatibility import is_dataclass, dataclass_fields
-
-from .core import quote
+from .base import (
+    DaskMethodsMixin,
+    dont_optimize,
+    is_dask_collection,
+    replace_name_in_key,
+)
+from .base import tokenize as _tokenize
 from .context import globalmethod
-from .optimization import cull
-from .utils import funcname, methodcaller, OperatorMethodMixin, ensure_dict, apply
+from .core import quote
 from .highlevelgraph import HighLevelGraph
+from .optimization import cull
+from .utils import OperatorMethodMixin, apply, ensure_dict, funcname, methodcaller
 
 __all__ = ["Delayed", "delayed"]
 
@@ -63,16 +67,16 @@ def unpack_collections(expr):
     >>> a = delayed(1, 'a')
     >>> b = delayed(2, 'b')
     >>> task, collections = unpack_collections([a, b, 3])
-    >>> task  # doctest: +SKIP
+    >>> task
     ['a', 'b', 3]
-    >>> collections  # doctest: +SKIP
-    (a, b)
+    >>> collections
+    (Delayed('a'), Delayed('b'))
 
     >>> task, collections = unpack_collections({a: 1, b: 2})
-    >>> task  # doctest: +SKIP
-    (dict, [['a', 1], ['b', 2]])
-    >>> collections  # doctest: +SKIP
-    {a, b}
+    >>> task
+    (<class 'dict'>, [['a', 1], ['b', 2]])
+    >>> collections
+    (Delayed('a'), Delayed('b'))
     """
     if isinstance(expr, Delayed):
         return expr._key, (expr,)
@@ -105,7 +109,11 @@ def unpack_collections(expr):
 
     if is_dataclass(expr):
         args, collections = unpack_collections(
-            [[f.name, getattr(expr, f.name)] for f in dataclass_fields(expr)]
+            [
+                [f.name, getattr(expr, f.name)]
+                for f in fields(expr)
+                if hasattr(expr, f.name)  # if init=False, field might not exist
+            ]
         )
 
         return (apply, typ, (), (dict, args)), collections
@@ -183,7 +191,11 @@ def to_task_dask(expr):
 
     if is_dataclass(expr):
         args, dsk = to_task_dask(
-            [[f.name, getattr(expr, f.name)] for f in dataclass_fields(expr)]
+            [
+                [f.name, getattr(expr, f.name)]
+                for f in fields(expr)
+                if hasattr(expr, f.name)  # if init=False, field might not exist
+            ]
         )
 
         return (apply, typ, (), (dict, args)), dsk
@@ -272,7 +284,7 @@ def delayed(obj, name=None, pure=None, nout=None, traverse=True):
     11
 
     >>> x = delayed(inc, pure=True)(10)
-    >>> type(x) == Delayed              # doctest: +SKIP
+    >>> type(x) == Delayed
     True
     >>> x.compute()
     11
@@ -346,7 +358,7 @@ def delayed(obj, name=None, pure=None, nout=None, traverse=True):
     ``delayed`` can also be applied to objects to make operations on them lazy:
 
     >>> a = delayed([1, 2, 3])
-    >>> isinstance(a, Delayed)      # doctest: +SKIP
+    >>> isinstance(a, Delayed)
     True
     >>> a.compute()
     [1, 2, 3]
@@ -366,14 +378,14 @@ def delayed(obj, name=None, pure=None, nout=None, traverse=True):
     Delayed results act as a proxy to the underlying object. Many operators
     are supported:
 
-    >>> (a + [1, 2]).compute()      # doctest: +SKIP
+    >>> (a + [1, 2]).compute()
     [1, 2, 3, 1, 2]
-    >>> a[1].compute()              # doctest: +SKIP
+    >>> a[1].compute()
     2
 
     Method and attribute access also works:
 
-    >>> a.count(2).compute()        # doctest: +SKIP
+    >>> a.count(2).compute()
     1
 
     Note that if a method doesn't exist, no error will be thrown until runtime:
@@ -431,25 +443,23 @@ def delayed(obj, name=None, pure=None, nout=None, traverse=True):
         task = quote(obj)
         collections = set()
 
+    if not (nout is None or (type(nout) is int and nout >= 0)):
+        raise ValueError("nout must be None or a non-negative integer, got %s" % nout)
     if task is obj:
-        if not (nout is None or (type(nout) is int and nout >= 0)):
-            raise ValueError(
-                "nout must be None or a non-negative integer, got %s" % nout
-            )
         if not name:
             try:
                 prefix = obj.__name__
             except AttributeError:
                 prefix = type(obj).__name__
             token = tokenize(obj, nout, pure=pure)
-            name = "%s-%s" % (prefix, token)
+            name = f"{prefix}-{token}"
         return DelayedLeaf(obj, name, pure=pure, nout=nout)
     else:
         if not name:
-            name = "%s-%s" % (type(obj).__name__, tokenize(task, pure=pure))
+            name = f"{type(obj).__name__}-{tokenize(task, pure=pure)}"
         layer = {name: task}
         graph = HighLevelGraph.from_collections(name, layer, dependencies=collections)
-        return Delayed(name, graph)
+        return Delayed(name, graph, nout)
 
 
 def right(method):
@@ -519,7 +529,7 @@ class Delayed(DaskMethodsMixin, OperatorMethodMixin):
         return Delayed(key, dsk, self._length)
 
     def __repr__(self):
-        return "Delayed({0})".format(repr(self.key))
+        return f"Delayed({repr(self.key)})"
 
     def __hash__(self):
         return hash(self.key)
@@ -597,7 +607,7 @@ def call_function(func, func_token, args, kwargs, pure=None, nout=None):
     pure = kwargs.pop("pure", pure)
 
     if dask_key_name is None:
-        name = "%s-%s" % (
+        name = "{}-{}".format(
             funcname(func),
             tokenize(func_token, *args, pure=pure, **kwargs),
         )
@@ -709,5 +719,5 @@ except AttributeError:
 
 
 def single_key(seq):
-    """ Pick out the only element of this list, a list of keys """
+    """Pick out the only element of this list, a list of keys"""
     return seq[0]
