@@ -1,16 +1,20 @@
 import copyreg
 import multiprocessing
+import multiprocessing.pool
 import os
 import pickle
 import sys
 import traceback
+from concurrent.futures import ProcessPoolExecutor
 from functools import partial
 from warnings import warn
 
+import cloudpickle
+
 from . import config
+from .local import MultiprocessingPoolExecutor, get_async, reraise
+from .optimization import cull, fuse
 from .system import CPU_COUNT
-from .local import reraise, get_async  # TODO: get better get
-from .optimization import fuse, cull
 from .utils import ensure_dict
 
 
@@ -21,23 +25,8 @@ def _reduce_method_descriptor(m):
 # type(set.union) is used as a proxy to <class 'method_descriptor'>
 copyreg.pickle(type(set.union), _reduce_method_descriptor)
 
-
-try:
-    import cloudpickle
-
-    _dumps = partial(cloudpickle.dumps, protocol=pickle.HIGHEST_PROTOCOL)
-    _loads = cloudpickle.loads
-except ImportError:
-
-    def _dumps(obj, **kwargs):
-        try:
-            return pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL, **kwargs)
-        except (pickle.PicklingError, AttributeError) as exc:
-            raise ModuleNotFoundError(
-                "Please install cloudpickle to use the multiprocessing scheduler"
-            ) from exc
-
-    _loads = pickle.loads
+_dumps = partial(cloudpickle.dumps, protocol=pickle.HIGHEST_PROTOCOL)
+_loads = cloudpickle.loads
 
 
 def _process_get_id():
@@ -84,7 +73,7 @@ exceptions = dict()
 
 
 def remote_exception(exc, tb):
-    """ Metaclass that wraps exception type in RemoteException """
+    """Metaclass that wraps exception type in RemoteException"""
     if type(exc) in exceptions:
         typ = exceptions[type(exc)]
         return typ(exc, tb)
@@ -139,7 +128,7 @@ and on Windows, because they each only support a single context.
 
 
 def get_context():
-    """ Return the current multiprocessing context."""
+    """Return the current multiprocessing context."""
     # fork context does fork()-without-exec(), which can lead to deadlocks,
     # so default to "spawn".
     context_name = config.get("multiprocessing.context", "spawn")
@@ -160,7 +149,8 @@ def get(
     func_dumps=None,
     optimize_graph=True,
     pool=None,
-    **kwargs
+    chunksize=None,
+    **kwargs,
 ):
     """Multiprocessed get function appropriate for Bags
 
@@ -173,14 +163,19 @@ def get(
     num_workers : int
         Number of worker processes (defaults to number of cores)
     func_dumps : function
-        Function to use for function serialization
-        (defaults to cloudpickle.dumps if available, otherwise pickle.dumps)
+        Function to use for function serialization (defaults to cloudpickle.dumps)
     func_loads : function
-        Function to use for function deserialization
-        (defaults to cloudpickle.loads if available, otherwise pickle.loads)
+        Function to use for function deserialization (defaults to cloudpickle.loads)
     optimize_graph : bool
         If True [default], `fuse` is applied to the graph before computation.
+    pool : Executor or Pool
+        Some sort of `Executor` or `Pool` to use
+    chunksize: int, optional
+        Size of chunks to use when dispatching work.
+        Defaults to 5 as some batching is helpful.
+        If -1, will be computed to evenly divide ready work across workers.
     """
+    chunksize = chunksize or config.get("chunksize", 6)
     pool = pool or config.get("pool", None)
     num_workers = num_workers or config.get("num_workers", None) or CPU_COUNT
     if pool is None:
@@ -194,9 +189,13 @@ def get(
             # https://github.com/dask/dask/issues/6640.
             os.environ["PYTHONHASHSEED"] = "6640"
         context = get_context()
-        pool = context.Pool(num_workers, initializer=initialize_worker_process)
+        pool = ProcessPoolExecutor(
+            num_workers, mp_context=context, initializer=initialize_worker_process
+        )
         cleanup = True
     else:
+        if isinstance(pool, multiprocessing.pool.Pool):
+            pool = MultiprocessingPoolExecutor(pool)
         cleanup = False
 
     # Optimize Dask
@@ -218,8 +217,8 @@ def get(
     try:
         # Run
         result = get_async(
-            pool.apply_async,
-            len(pool._pool),
+            pool.submit,
+            pool._max_workers,
             dsk3,
             keys,
             get_id=_process_get_id,
@@ -227,11 +226,12 @@ def get(
             loads=loads,
             pack_exception=pack_exception,
             raise_exception=reraise,
-            **kwargs
+            chunksize=chunksize,
+            **kwargs,
         )
     finally:
         if cleanup:
-            pool.close()
+            pool.shutdown()
     return result
 
 
