@@ -4,6 +4,7 @@ import subprocess
 import sys
 import time
 from collections import OrderedDict
+from concurrent.futures import Executor
 from operator import add, mul
 
 import pytest
@@ -128,9 +129,8 @@ def test_tokenize_numpy_scalar_string_rep():
 
 @pytest.mark.skipif("not np")
 def test_tokenize_numpy_array_on_object_dtype():
-    assert tokenize(np.array(["a", "aa", "aaa"], dtype=object)) == tokenize(
-        np.array(["a", "aa", "aaa"], dtype=object)
-    )
+    a = np.array(["a", "aa", "aaa"], dtype=object)
+    assert tokenize(a) == tokenize(a)
     assert tokenize(np.array(["a", None, "aaa"], dtype=object)) == tokenize(
         np.array(["a", None, "aaa"], dtype=object)
     )
@@ -138,6 +138,17 @@ def test_tokenize_numpy_array_on_object_dtype():
         np.array([(1, "a"), (1, None), (1, "aaa")], dtype=object)
     ) == tokenize(np.array([(1, "a"), (1, None), (1, "aaa")], dtype=object))
 
+    # Trigger non-deterministic hashing for object dtype
+    class NoPickle:
+        pass
+
+    x = np.array(["a", None, NoPickle], dtype=object)
+    assert tokenize(x) != tokenize(x)
+
+    with dask.config.set({"tokenize.ensure-deterministic": True}):
+        with pytest.raises(RuntimeError, match="cannot be deterministically hashed"):
+            tokenize(x)
+
 
 @pytest.mark.skipif("not np")
 def test_tokenize_numpy_memmap_offset(tmpdir):
@@ -181,7 +192,7 @@ def test_tokenize_numpy_memmap():
         b = tokenize(mm[1, :])
         c = tokenize(mm[0:3, :])
         d = tokenize(mm[:, 0])
-        assert len(set([a, b, c, d])) == 4
+        assert len({a, b, c, d}) == 4
         assert tokenize(mm) == tokenize(mm2)
         assert tokenize(mm[1, :]) == tokenize(mm2[1, :])
 
@@ -226,6 +237,23 @@ def test_normalize_base():
         assert normalize_token(i) is i
 
 
+def test_tokenize_object():
+    o = object()
+    # Defaults to non-deterministic tokenization
+    assert normalize_token(o) != normalize_token(o)
+
+    with dask.config.set({"tokenize.ensure-deterministic": True}):
+        with pytest.raises(RuntimeError, match="cannot be deterministically hashed"):
+            normalize_token(o)
+
+
+def test_tokenize_callable():
+    def my_func(a, b, c=1):
+        return a + b + c
+
+    assert tokenize(my_func) == tokenize(my_func)  # Consistent token
+
+
 @pytest.mark.skipif("not pd")
 def test_tokenize_pandas():
     a = pd.DataFrame({"x": [1, 2, 3], "y": ["4", "asd", None]}, index=[1, 2, 3])
@@ -255,7 +283,7 @@ def test_tokenize_pandas_invalid_unicode():
 @pytest.mark.skipif("not pd")
 def test_tokenize_pandas_mixed_unicode_bytes():
     df = pd.DataFrame(
-        {"ö".encode("utf8"): [1, 2, 3], "ö": ["ö", "ö".encode("utf8"), None]},
+        {"ö".encode(): [1, 2, 3], "ö": ["ö", "ö".encode(), None]},
         index=[1, 2, 3],
     )
     tokenize(df)
@@ -335,6 +363,10 @@ def test_tokenize_method():
     assert tokenize(a) == tokenize(a)
     assert tokenize(a) != tokenize(b)
 
+    for ensure in [True, False]:
+        with dask.config.set({"tokenize.ensure-deterministic": ensure}):
+            assert tokenize(a) == tokenize(a)
+
     # dispatch takes precedence
     before = tokenize(a)
     normalize_token.register(Foo, lambda self: self.x + 1)
@@ -439,12 +471,16 @@ def test_tokenize_dense_sparse_array(cls_name):
     assert tokenize(a) != tokenize(b)
 
 
-def test_tokenize_object_with_recursion_error_returns_uuid():
+def test_tokenize_object_with_recursion_error():
     cycle = dict(a=None)
     cycle["a"] = cycle
 
     assert len(tokenize(cycle)) == 32
 
+    with dask.config.set({"tokenize.ensure-deterministic": True}):
+        with pytest.raises(RuntimeError, match="cannot be deterministically hashed"):
+            tokenize(cycle)
+
 
 def test_tokenize_datetime_date():
     # Same date
@@ -892,7 +928,7 @@ def test_compute_array_dataframe():
 
 @pytest.mark.skipif("not dd")
 def test_compute_dataframe_valid_unicode_in_bytes():
-    df = pd.DataFrame(data=np.random.random((3, 1)), columns=["ö".encode("utf8")])
+    df = pd.DataFrame(data=np.random.random((3, 1)), columns=["ö".encode()])
     dd.from_pandas(df, npartitions=4)
 
 
@@ -1344,12 +1380,17 @@ def test_raise_get_keyword():
     assert "scheduler=" in str(info.value)
 
 
+class MyExecutor(Executor):
+    _max_workers = None
+
+
 def test_get_scheduler():
     assert get_scheduler() is None
     assert get_scheduler(scheduler=dask.local.get_sync) is dask.local.get_sync
     assert get_scheduler(scheduler="threads") is dask.threaded.get
     assert get_scheduler(scheduler="sync") is dask.local.get_sync
     assert callable(get_scheduler(scheduler=dask.local.synchronous_executor))
+    assert callable(get_scheduler(scheduler=MyExecutor()))
     with dask.config.set(scheduler="threads"):
         assert get_scheduler() is dask.threaded.get
     assert get_scheduler() is None
