@@ -4,37 +4,43 @@ import operator
 import pickle
 
 import pytest
+from tlz import curry
 
 from dask import get
-from dask.utils import (
-    getargspec,
-    takes_multiple_arguments,
-    Dispatch,
-    random_state_data,
-    memory_repr,
-    methodcaller,
-    M,
-    skip_doctest,
-    SerializableLock,
-    funcname,
-    ndeepmap,
-    ensure_dict,
-    extra_titles,
-    asciitable,
-    itemgetter,
-    partial_by_order,
-    has_keyword,
-    derived_from,
-    parse_timedelta,
-    parse_bytes,
-    is_arraylike,
-    iter_chunks,
-    stringify,
-    stringify_collection_keys,
-)
-from dask.utils_test import inc
 from dask.highlevelgraph import HighLevelGraph
 from dask.optimization import SubgraphCallable
+from dask.utils import (
+    Dispatch,
+    M,
+    SerializableLock,
+    _deprecated,
+    asciitable,
+    derived_from,
+    ensure_dict,
+    extra_titles,
+    format_bytes,
+    funcname,
+    getargspec,
+    has_keyword,
+    ignoring,
+    is_arraylike,
+    itemgetter,
+    iter_chunks,
+    memory_repr,
+    methodcaller,
+    ndeepmap,
+    noop_context,
+    parse_bytes,
+    parse_timedelta,
+    partial_by_order,
+    random_state_data,
+    skip_doctest,
+    stringify,
+    stringify_collection_keys,
+    takes_multiple_arguments,
+    typename,
+)
+from dask.utils_test import inc
 
 
 def test_getargspec():
@@ -98,7 +104,7 @@ def test_dispatch():
     foo.register(tuple, lambda a: tuple(foo(i) for i in a))
 
     def f(a):
-        """ My Docstring """
+        """My Docstring"""
         return a
 
     foo.register(object, f)
@@ -155,6 +161,38 @@ def test_dispatch_lazy():
     assert foo(1) == 1
 
 
+def test_dispatch_lazy_walks_mro():
+    """Check that subclasses of classes with lazily registered handlers still
+    use their parent class's handler by default"""
+    import decimal
+
+    class Lazy(decimal.Decimal):
+        pass
+
+    class Eager(Lazy):
+        pass
+
+    foo = Dispatch()
+
+    @foo.register(Eager)
+    def eager_handler(x):
+        return "eager"
+
+    def lazy_handler(a):
+        return "lazy"
+
+    @foo.register_lazy("decimal")
+    def register_decimal():
+        foo.register(decimal.Decimal, lazy_handler)
+
+    assert foo.dispatch(Lazy) == lazy_handler
+    assert foo(Lazy(1)) == "lazy"
+    assert foo.dispatch(decimal.Decimal) == lazy_handler
+    assert foo(decimal.Decimal(1)) == "lazy"
+    assert foo.dispatch(Eager) == eager_handler
+    assert foo(Eager(1)) == "eager"
+
+
 def test_random_state_data():
     np = pytest.importorskip("numpy")
     seed = 37
@@ -363,9 +401,7 @@ def test_funcname_long():
 
 
 def test_funcname_toolz():
-    toolz = pytest.importorskip("tlz")
-
-    @toolz.curry
+    @curry
     def foo(a, b, c):
         pass
 
@@ -638,3 +674,94 @@ def test_stringify_collection_keys():
     assert res[0] == str(obj[0])
     assert res[1] == str(obj[1])
     assert res[2] == obj[2]
+
+
+@pytest.mark.parametrize(
+    "n,expect",
+    [
+        (0, "0 B"),
+        (920, "920 B"),
+        (930, "0.91 kiB"),
+        (921.23 * 2 ** 10, "921.23 kiB"),
+        (931.23 * 2 ** 10, "0.91 MiB"),
+        (921.23 * 2 ** 20, "921.23 MiB"),
+        (931.23 * 2 ** 20, "0.91 GiB"),
+        (921.23 * 2 ** 30, "921.23 GiB"),
+        (931.23 * 2 ** 30, "0.91 TiB"),
+        (921.23 * 2 ** 40, "921.23 TiB"),
+        (931.23 * 2 ** 40, "0.91 PiB"),
+        (2 ** 60, "1024.00 PiB"),
+    ],
+)
+def test_format_bytes(n, expect):
+    assert format_bytes(int(n)) == expect
+
+
+def test_deprecated():
+    @_deprecated()
+    def foo():
+        return "bar"
+
+    with pytest.warns(FutureWarning) as record:
+        assert foo() == "bar"
+
+    assert len(record) == 1
+    msg = str(record[0].message)
+    assert "foo is deprecated" in msg
+    assert "removed in a future release" in msg
+
+
+def test_deprecated_version():
+    @_deprecated(version="1.2.3")
+    def foo():
+        return "bar"
+
+    with pytest.warns(FutureWarning, match="deprecated in version 1.2.3"):
+        assert foo() == "bar"
+
+
+def test_deprecated_category():
+    @_deprecated(category=DeprecationWarning)
+    def foo():
+        return "bar"
+
+    with pytest.warns(DeprecationWarning):
+        assert foo() == "bar"
+
+
+def test_deprecated_message():
+    @_deprecated(message="woohoo")
+    def foo():
+        return "bar"
+
+    with pytest.warns(FutureWarning) as record:
+        assert foo() == "bar"
+
+    assert len(record) == 1
+    assert str(record[0].message) == "woohoo"
+
+
+def test_ignoring_deprecated():
+    with pytest.warns(FutureWarning, match="contextlib.suppress"):
+        with ignoring(ValueError):
+            pass
+
+
+def test_noop_context_deprecated():
+    with pytest.warns(FutureWarning, match="contextlib.nullcontext"):
+        with noop_context():
+            pass
+
+
+def test_typename():
+    assert typename(HighLevelGraph) == "dask.highlevelgraph.HighLevelGraph"
+    assert typename(HighLevelGraph, short=True) == "dask.HighLevelGraph"
+
+
+class MyType:
+    pass
+
+
+def test_typename_on_instances():
+    instance = MyType()
+    assert typename(instance) == typename(MyType)
